#!/usr/bin/perl5

########################################################################
# bnbanner.pl - program for printing bengali banner
# Author : Abhijit Das (Barda) [abhij@csa.iisc.ernet.in]
# Last modified : Fri Oct 17 1997
########################################################################

$INSTALLDIR = "/usr/local/lib/bnbanner";
$IMGDIR = "$INSTALLDIR/asc";
$DPI = 250;

sub readenc {
   my $line;

   $ENCFILE = "$INSTALLDIR/enc/bn$DPI.enc";
   open(ENCF,"<$ENCFILE");
   while ($line = <ENCF>) {
      unless ($line =~ /^#/) {
         chop $line;
         if ($line =~ /(^(a|e|i|o|u|A|E|I|O|U|R|L)+?)=(.+)$/) {
            $line =~ /^(.+?)=(.*?):(.*?):(.*?):(.*?):(.*?)$/;
            $map{$1} = $2;
            $map{$1 . "@"} = $3;
            $map{"@" . $1} = $4;
            $len{$1 . "@"} = $5;
            $len{"@" . $1} = $6;
         } elsif ($line =~ /(^.+?)=(.+)$/) {
            $map{$1} = $2;
         }
      }
   }
}

sub vowel ($) {
   return ($_[0] =~ /^(a|e|i|o|u|A|E|I|O|U|R|L)/);
}

sub consonant ($) {
   if (vowel($_[0])) { return 0; }
   if ($_[0] =~ /^\d/) { return 0; }
   if ($_[0] =~ /^_/) { return 0; }
   if ($_[0] =~ /^(\w|\^)/) { return 1; }
   return 0;
}

sub findtokens ($) {
   my $line = $_[0];
   my $i = 0;
   my @token;
   my ($swar, $banjon);

   while (length($line) > 0) {
      if ($line =~ s/^(a|e|i|o|u|A|E|I|O|U|R|L)+//) {
         $token[$i] = $&; $i++;
      } elsif (consonant $line) {
         $banjon = "";
         while (consonant $line) {
            $banjon .= substr($line, 0, 1);
            $line = substr($line, 1);
         }
         $line =~ s/^(a|e|i|o|u|A|E|I|O|U|R|L)*//;
         $swar = $&;
         if ($swar eq "") {
            $token[$i] = $banjon; $i++;
         } else {
            $token[$i] = $banjon;
            $token[$i] .= "&" . $swar;
            $i++;
         }
      } elsif ($line =~ s/^__//) {
         $token[$i] = "__"; $i++;
      } elsif ($line =~ s/^_//) {
      } else {
         $token[$i] = substr($line, 0, 1); $i++;
         $line = substr($line, 1);
      }
   }

   return @token;
}

sub printtokens (@) {
   my @token = @_;
   my ($i, $j, $k);
   my $out = "";
   my (@acc, @next);
   my ($swar, $banjon, $line, $prelen, $postlen, $linelen);
   my ($ref, $rafala, $jafala, $chandra);

   if (@token == 0) { return ""; }
   for ($i = 0; $i < @token; $i++) {
      @next = ();
      if ($token[$i] =~ /^(a|e|i|o|u|A|E|I|O|U|R|L)+$/) {
         $map = $map{$token[$i]};
         if ($map eq "") {
            print STDERR "unknown vowel code : " . $token[$i] . " ... omitting\n";
         } else {
            $fname = "$IMGDIR/$DPI/$map.txt";
            open(CHF, "<$fname");
            $j = 0;
            while ($line = <CHF>) {
               chop $line; $next[$j] = $line; $j++;
            }
            close(CHF);
         }
      } elsif (consonant $token[$i]) {
         $ref = $rafala = $jafala = $chandra = 0;
         if ( ($token[$i] eq "^") || ($token[$i] eq "^j") || ($token[$i] eq "^y") ) {
            if ($token[$i] eq "^") {
               $fname = "$IMGDIR/$DPI/119.txt";
            } else {
               $fname = "$IMGDIR/$DPI/13.txt";
            }
            open(CHF, "<$fname");
            $j = 0;
            while ($line = <CHF>) {
               chop $line;
               $next[$j] = $line;
               $j++;
            }
            close(CHF);
         } elsif ($token[$i] =~ /&/) {
            $token[$i] =~ /^(.+)&(.+)$/;
            $banjon = $1; $swar = $2;
            while ($banjon =~ s/^\^r//) { $ref = 1; }
            do {
               $done = 1;
               if ($banjon =~ s/\^$//) { $chandra = 1; $done = 0; }
               if ($banjon =~ s/\^j$//) { $jafala = 1; $done = 0; }
               if ($banjon =~ s/\^y$//) { $jafala = 1; $done = 0; }
               if ($banjon =~ s/\^r$//) { $rafala = 1; $done = 0; }
            } while (! $done);
            $map = $map{$banjon};
            if ($map eq "") {
               print STDERR "unknown consonant code : " . $banjon . " ... omitting\n";
               $map = 32;
            }
            $prelen = $len{$swar . "@"}; $postlen = $len{"@" . $swar};
            $prelen = 0 if ($prelen eq "");
            $postlen = 0 if ($postlen eq "");
            $fname = "$IMGDIR/$DPI/$map.txt";
            open(CHF, "<$fname");
            $j = 0;
            while ($line = <CHF>) {
               chop $line; $banlen = length($line);
               $next[$j] = (' ' x $prelen) . $line . (' ' x $postlen);
               $j++;
            }
            close(CHF);

            $map = $map{$swar . "@"};
            unless ($map eq "") {
               $fname = "$IMGDIR/$DPI/$map.txt";
               open(CHF, "<$fname");
               $j = 0;
               while ($line = <CHF>) {
                  chop $line;
                  $linelen = length $line;
                  for ($k=0; $k < $linelen; $k++) {
                     if (substr($line,$k,1) ne " ") {
                        substr($next[$j],$k,1) = substr($line,$k,1);
                     }
                  }
                  $j++;
               }
               close(CHF);
            }
            if ($jafala) {
               $fname = "$IMGDIR/$DPI/13.txt";
               open(CHF,"<$fname");
               $j = 0;
               while ($line = <CHF>) {
                  chop $line;
                  substr($next[$j], length($next[$j])-$postlen,0) .= $line;
                  $j++;
               }
               close(CHF);
            }
            $map = $map{"@" . $swar};
            unless ($map eq "") {
               $fname = "$IMGDIR/$DPI/$map.txt";
               open(CHF, "<$fname");
               $j = 0;
               while ($line = <CHF>) {
                  chop $line;
                  $linelen = length $line;
                  for ($k=0; $k < $linelen; $k++) {
                     if (substr($line,$k,1) ne " ") {
                        substr($next[$j], length($next[$j])-$linelen+$k, 1) = substr($line,$k,1);
                     }
                  }
                  $j++;
               }
               close(CHF);
            }
         } else {
            $prelen = $postlen = 0;
            $banjon = $token[$i];
            while ($banjon =~ s/^\^r//) { $ref = 1; }
            do {
               $done = 1;
               if ($banjon =~ s/\^$//) { $chandra = 1; $done = 0; }
               if ($banjon =~ s/\^j$//) { $jafala = 1; $done = 0; }
               if ($banjon =~ s/\^y$//) { $jafala = 1; $done = 0; }
               if ($banjon =~ s/\^r$//) { $rafala = 1; $done = 0; }
            } while (! $done);
            $map = $map{$banjon};
            if ($map eq "") {
               print STDERR "unknown consonant code : " . $token[$i] . " ... omitting\n";
            } else {
               $fname = "$IMGDIR/$DPI/$map.txt";
               open(CHF, "<$fname");
               $j = 0;
               while ($line = <CHF>) {
                  chop $line; $banlen = length($line);
                  $next[$j] = $line; $j++;
               }
               close(CHF);
            }
            if ($jafala) {
               $fname = "$IMGDIR/$DPI/13.txt";
               open(CHF,"<$fname");
               $j = 0;
               while ($line = <CHF>) {
                  chop $line;
                  $next[$j] .= $line;
                  $j++;
               }
               close(CHF);
            }
         }
         if ($chandra) {
            $fname = "$IMGDIR/$DPI/119.txt";
            open(CHF,"<$fname");
            $j = 0;
            while ($line = <CHF>) {
               chop $line;
               $linelen = length $line;
               for ($k=0; $k < $linelen; $k++) {
                  if (substr($line,$k,1) ne " ") {
                     substr($next[$j], length($next[$j])-$linelen+$k, 1) = substr($line,$k,1);
                  }
               }
               $j++;
            }
            close(CHF);
         }
         if ($rafala) {
            $fname = "$IMGDIR/$DPI/15.txt";
            open(CHF,"<$fname");
            $j = 0;
            while ($line = <CHF>) {
               chop $line;
               $linelen = length $line;
               for ($k=0; $k < $linelen; $k++) {
                  if (substr($line,$k,1) ne " ") {
                     substr($next[$j], $prelen + $banlen - length($line) + $k - 2, 1) = substr($line,$k,1);
                  }
               }
               $j++;
            }
            close(CHF);
         }
         if ($ref) {
            $fname = "$IMGDIR/$DPI/14.txt";
            open(CHF,"<$fname");
            $j = 0;
            while ($line = <CHF>) {
               chop $line;
               $next[$j] .= $line;
               $j++;
            }
            close(CHF);
         }
      } else {
         if ($token[$i] eq "__") {
            $fname = "$IMGDIR/$DPI/12.txt";
         } else {
            $fname = "$IMGDIR/$DPI/" . ord($token[$i]) . ".txt";
         }
         open(CHF, "<$fname");
         $j = 0;
         while ($line = <CHF>) {
            chop $line; $next[$j] = $line; $j++;
         }
         close(CHF);
      }

      for ($k=0; $k < @next; $k++) {
         if (defined $acc[$k]) {
            $acc[$k] .= $next[$k];
         } else {
            $acc[$k] = $next[$k];
         }
      }
   }

   for ($k=0; $k < @acc - 1; $k++) {
      print $acc[$k] . "\n";
   }
}

if ( ($ARGV[0] eq "-help") || ($ARGV[0] eq "-h") ) {
print <<EOM;
bnbanner: program for printing bengali banners in stdout
Usage:
  bnbanner [-h|-help]
                 prints this message
  bnbanner [-small|-normal|-large|-huge|-gigantic] string1 [string2 ...]
                 prints banner corresponding to string1, string2, ...
                 in the specified size (default size: normal)
EOM
exit;
}

if ($ARGV[0] eq "-small") {
   shift(@ARGV);
   $DPI = 208;
} elsif ($ARGV[0] eq "-normal") {
   shift(@ARGV);
} elsif ($ARGV[0] eq "-large") {
   shift(@ARGV);
   $DPI = 300;
} elsif ($ARGV[0] eq "-huge") {
   shift(@ARGV);
   $DPI = 360;
} elsif ($ARGV[0] eq "-gigantic") {
   shift(@ARGV);
   $DPI = 432;
}

readenc;

while ($nextword = shift(@ARGV)) {
   @token = findtokens $nextword;
   printtokens @token;
}

####### End of bnbanner.pl #######
